@@ -175,16 +175,24 @@ fn mutate_error_state_f(mut task: FTask) {
     }
 }
 
-fn increment_result_lock_s(task: &STask2) {
+fn increment_result_lock_s(task: &mut STask2) {
     let (lock, cond) = &*task.result_height;
     let mut result_height = lock.lock().unwrap();
+    {
+        let err = task.error_state.borrow_mut();
+        err.fetch_or(true, Ordering::SeqCst);
+    }
     *result_height += 1;
     cond.notify_all();
 }
 
-fn increment_result_lock_f(task: &FTask2) {
+fn increment_result_lock_f(task: &mut FTask2) {
     let (lock, cond) = &*task.result_height;
     let mut result_height = lock.lock().unwrap();
+    {
+        let err = task.error_state.borrow_mut();
+        err.fetch_or(true, Ordering::SeqCst);
+    }
     *result_height += 1;
     cond.notify_all();
 }
@@ -401,8 +409,11 @@ fn fetch_fblock_connected(
                     // this block ends task in waiting in the first period
                     if task.error_state.load(Ordering::SeqCst) {
                         *outputs_insertion_height += 1;
-                        // may be the later tasks are waiting in the second lock, increment lock
-                        increment_result_lock_f(&task);
+                        cond.notify_all();
+                        let (lock, cond2) = &*task.result_height;
+                        let mut result_height = lock.lock().unwrap();
+                        *result_height += 1;
+                        cond2.notify_all();
                         return;
                     }
                     *outputs_insertion_height += 1;
@@ -452,12 +463,8 @@ fn fetch_fblock_connected(
                                 output_tx.input.push(out);
                             } else {
                                 warn!("cannot find previous outpoint, bad data");
-                                {
-                                    let err = task.error_state.borrow_mut();
-                                    err.fetch_or(true, Ordering::SeqCst);
-                                }
                                 // only increment result lock
-                                increment_result_lock_f(&task);
+                                increment_result_lock_f(&mut task);
                                 return;
                             }
                         } else {
@@ -467,7 +474,7 @@ fn fetch_fblock_connected(
                                 err.fetch_or(true, Ordering::SeqCst);
                             }
                             // only increment result lock
-                            increment_result_lock_f(&task);
+                            increment_result_lock_f(&mut task);
                             return;
                         }
                     }
@@ -564,9 +571,13 @@ fn fetch_sblock_connected(
                     }
                     // this block ends task in waiting in the first period
                     if task.error_state.load(Ordering::SeqCst) {
-                        *outputs_insertion_height += 1;
                         // may be the later tasks are waiting in the second lock, increment lock
-                        increment_result_lock_s(&task);
+                        *outputs_insertion_height += 1;
+                        cond.notify_all();
+                        let (lock, cond2) = &*task.result_height;
+                        let mut result_height = lock.lock().unwrap();
+                        *result_height += 1;
+                        cond2.notify_all();
                         return;
                     }
                     *outputs_insertion_height += 1;
@@ -618,7 +629,7 @@ fn fetch_sblock_connected(
                                     err.fetch_or(true, Ordering::SeqCst);
                                 }
                                 // only increment result lock
-                                increment_result_lock_s(&task);
+                                increment_result_lock_s(&mut task);
                                 return;
                             }
                         } else {
@@ -628,7 +639,7 @@ fn fetch_sblock_connected(
                                 err.fetch_or(true, Ordering::SeqCst);
                             }
                             // only increment result lock
-                            increment_result_lock_s(&task);
+                            increment_result_lock_s(&mut task);
                             return;
                         }
                     }
